﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;

namespace howto_fill_circle_overlap
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The three circles.
        private List<RectangleF> Circles = new List<RectangleF>();

        // The triangle and centroid.
        private PointF[] Triangle = null;
        private PointF Centroid;
        private GraphicsPath ArcPath = null;

        private bool Drawing = false;
        private PointF StartPoint, EndPoint;

        private void picCanvas_MouseDown(object sender, MouseEventArgs e)
        {
            if (Circles.Count == 3)
            {
                Circles = new List<RectangleF>();
                Triangle = null;
                ArcPath = null;
            }

            StartPoint = e.Location;
            EndPoint = e.Location;
            Drawing = true;
        }

        private void picCanvas_MouseMove(object sender, MouseEventArgs e)
        {
            if (!Drawing) return;
            EndPoint = e.Location;
            picCanvas.Refresh();
        }

        private ArcInfo[] arc_infos;

        private void picCanvas_MouseUp(object sender, MouseEventArgs e)
        {
            if (!Drawing) return;
            Drawing = false;

            // If the start and end points are different,
            // add the new circle.
            if ((StartPoint.X != EndPoint.X) &&
                (StartPoint.X != EndPoint.X))
            {
                // Add the new circle.
                Circles.Add(PointsToRect(StartPoint, EndPoint));

                // Trilaterate.
                if (Circles.Count == 3)
                {
                    try
                    {
                        arc_infos = null;
                        Triangle = Trilaterate(
                            Circles[0], Circles[1], Circles[2],
                            out arc_infos);
                        Centroid = FindTriangleCentroid(
                            Triangle[0], Triangle[1], Triangle[2]);

                        // Make the arc path.
                        ArcPath = MakeArcPath(arc_infos);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                        Triangle = null;
                        ArcPath = null;
                    }
                }
            }

            picCanvas.Refresh();
        }

        // Make the arc path.
        private GraphicsPath MakeArcPath(ArcInfo[] arc_infos)
        {
            GraphicsPath path = new GraphicsPath();

            foreach (ArcInfo arc_info in arc_infos)
            {
                path.AddArc(arc_info.Rect, arc_info.StartAngle, arc_info.SweepAngle);
            }
            return path;
        }

        private void picCanvas_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.Clear(picCanvas.BackColor);
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;

            // If we have the overlap path, fill it.
            if (ArcPath != null)
            {
                // Fill the area.
                e.Graphics.FillPath(Brushes.Pink, ArcPath);

                // To see arcs that are out of order, draw the path
                // and don't draw the arcs in the next step.
                //using (Pen pen = new Pen(Color.Black, 3))
                //{
                //    e.Graphics.DrawPath(pen, ArcPath);
                //}

                // Draw the arcs.
                Color[] arc_colors = { Color.Red, Color.Green, Color.Blue };
                using (Pen pen = new Pen(Color.Black, 3))
                {
                    for (int i = 0; i < 3; i++)
                    {
                        pen.Color = arc_colors[i];
                        arc_infos[i].Draw(e.Graphics, pen);
                    }
                }
            }

            // If we have a triangle, draw it.
            if (Triangle != null)
            {
                // Fill the triangle.
                e.Graphics.FillPolygon(Brushes.Pink, Triangle);

                // Draw the POIs.
                foreach (PointF point in Triangle)
                {
                    e.Graphics.DrawPoint(Brushes.Black,
                        Pens.Black, point, 3);
                }

                // Fill the centroid.
                e.Graphics.DrawPoint(Brushes.LightGreen,
                    Pens.Green, Centroid, 3);
            }

            // Draw the circles.
            Pen[] pens = { Pens.Red, Pens.Green, Pens.Blue };
            for (int i = 0; i < Circles.Count; i++)
                e.Graphics.DrawEllipse(pens[i], Circles[i]);

            // If we're drawing a new circle, draw it.
            if (Drawing)
            {
                using (Pen pen = new Pen(Color.Black, 2))
                {
                    RectangleF rect = PointsToRect(StartPoint, EndPoint);
                    e.Graphics.DrawEllipse(pen, rect);

                    pen.Color = Color.White;
                    pen.DashPattern = new float[] { 5, 5 };
                    e.Graphics.DrawEllipse(pen, rect);
                }
            }
        }

        // Convert start and end points into a RectangleF.
        private RectangleF PointsToRect(PointF start_point, PointF end_point)
        {
            float dx = end_point.X - start_point.X;
            float dy = end_point.Y - start_point.Y;
            float radius = (float)Math.Sqrt(dx * dx + dy * dy) / 2f;
            float cx = (end_point.X + start_point.X) / 2f;
            float cy = (end_point.Y + start_point.Y) / 2f;
            return new RectangleF(
                cx - radius,
                cy - radius,
                2 * radius,
                2 * radius);
        }

        // Trilaterate.
        // Throw an exception if there is a problem.
        private PointF[] Trilaterate(RectangleF circle1,
            RectangleF circle2, RectangleF circle3,
            out ArcInfo[] arc_infos)
        {
            // Convert the circles from bounding rectangles
            // to centers and radii.
            float cx1 = circle1.X + circle1.Width / 2f;
            float cy1 = circle1.Y + circle1.Height / 2f;
            float cx2 = circle2.X + circle2.Width / 2f;
            float cy2 = circle2.Y + circle2.Height / 2f;
            float cx3 = circle3.X + circle3.Width / 2f;
            float cy3 = circle3.Y + circle3.Height / 2f;
            float r1 = circle1.Width / 2f;
            float r2 = circle2.Width / 2f;
            float r3 = circle3.Width / 2f;

            // Find the points of intersection.
            PointF
                intersection12a, intersection12b,
                intersection23a, intersection23b,
                intersection31a, intersection31b;
            if (FindCircleCircleIntersections(
                    cx1, cy1, r1, cx2, cy2, r2,
                    out intersection12a, out intersection12b) == 0)
                throw new Exception("circle1 and circle2 do not intersect.");
            if (FindCircleCircleIntersections(
                    cx2, cy2, r2, cx3, cy3, r3,
                    out intersection23a, out intersection23b) == 0)
                throw new Exception("circle2 and circle3 do not intersect.");
            if (FindCircleCircleIntersections(
                    cx3, cy3, r3, cx1, cy1, r1,
                    out intersection31a, out intersection31b) == 0)
                throw new Exception("circle3 and circle1 do not intersect.");

            // Find the points that make up the target area.
            PointF[] triangle = new PointF[3];
            PointF center1 = new PointF(cx1, cy1);
            PointF center2 = new PointF(cx2, cy2);
            PointF center3 = new PointF(cx3, cy3);
            if (Distance(intersection12a, center3) < Distance(intersection12b, center3))
                triangle[0] = intersection12a;
            else
                triangle[0] = intersection12b;
            if (Distance(intersection23a, center1) < Distance(intersection23b, center1))
                triangle[1] = intersection23a;
            else
                triangle[1] = intersection23b;
            if (Distance(intersection31a, center2) < Distance(intersection31b, center2))
                triangle[2] = intersection31a;
            else
                triangle[2] = intersection31b;

            // Make ArcInfo data.
            ArcInfo arc_infos1 = new ArcInfo(
                center1,
                triangle[0],
                triangle[2]);
            ArcInfo arc_infos2 = new ArcInfo(
                center2,
                triangle[0],
                triangle[1]);
            ArcInfo arc_infos3 = new ArcInfo(
                center3,
                triangle[1],
                triangle[2]);

            // Order the ArcInfo data.
            arc_infos = new ArcInfo[3];
            arc_infos[0] = arc_infos1;
            if (arc_infos1.Point2 == arc_infos2.Point1)
            {
                arc_infos[1] = arc_infos2;
                arc_infos[2] = arc_infos3;
            }
            else
            {
                arc_infos[1] = arc_infos3;
                arc_infos[2] = arc_infos2;
            }

            return triangle;
        }

        // Return the distance between two points.
        private float Distance(PointF point1, PointF point2)
        {
            float dx = point1.X - point2.X;
            float dy = point1.Y - point2.Y;
            return (float)Math.Sqrt(dx * dx + dy * dy);
        }

        // Determine where two circles intersect in C#
        // http://csharphelper.com/blog/2014/09/determine-where-two-circles-intersect-in-c/
        // Find the points where the two circles intersect.
        // Return the number of intersections.
        private int FindCircleCircleIntersections(
            float cx0, float cy0, float radius0,
            float cx1, float cy1, float radius1,
            out PointF intersection1, out PointF intersection2)
        {
            // Find the distance between the centers.
            float dx = cx0 - cx1;
            float dy = cy0 - cy1;
            double dist = Math.Sqrt(dx * dx + dy * dy);

            // See how many solutions there are.
            if (dist > radius0 + radius1)
            {
                // No solutions, the circles are too far apart.
                intersection1 = new PointF(float.NaN, float.NaN);
                intersection2 = new PointF(float.NaN, float.NaN);
                return 0;
            }
            else if (dist < Math.Abs(radius0 - radius1))
            {
                // No solutions, one circle contains the other.
                intersection1 = new PointF(float.NaN, float.NaN);
                intersection2 = new PointF(float.NaN, float.NaN);
                return 0;
            }
            else if ((dist == 0) && (radius0 == radius1))
            {
                // No solutions, the circles coincide.
                intersection1 = new PointF(float.NaN, float.NaN);
                intersection2 = new PointF(float.NaN, float.NaN);
                return 0;
            }
            else
            {
                // Find a and h.
                double a = (radius0 * radius0 -
                    radius1 * radius1 + dist * dist) / (2 * dist);
                double h = Math.Sqrt(radius0 * radius0 - a * a);

                // Find P2.
                double cx2 = cx0 + a * (cx1 - cx0) / dist;
                double cy2 = cy0 + a * (cy1 - cy0) / dist;

                // Get the points P3.
                intersection1 = new PointF(
                    (float)(cx2 + h * (cy1 - cy0) / dist),
                    (float)(cy2 - h * (cx1 - cx0) / dist));
                intersection2 = new PointF(
                    (float)(cx2 - h * (cy1 - cy0) / dist),
                    (float)(cy2 + h * (cx1 - cx0) / dist));

                // See if we have 1 or 2 solutions.
                if (dist == radius0 + radius1) return 1;
                return 2;
            }
        }

        // Return the triangle's centroid.
        private PointF FindTriangleCentroid(PointF p1, PointF p2, PointF p3)
        {
            return new PointF(
                (p1.X + p2.X + p3.X) / 3f,
                (p1.Y + p2.Y + p3.Y) / 3f);
        }
    }
}
